package coolCalendar;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A dialog box to edit an appointment.
 */
public class AppointmentDialog 
    extends JDialog
{
    public static final int OPEN_TYPE = 0;
    public static final int CLOSED_TYPE = 1;

    private static final String MINUTES = "minutes";
    private static final String HOURS = "hours";
    private static final String DAYS = "days";
    private static final String WEEKS = "weeks";
    private static final String MONTHS = "months";
    private static final String YEARS = "years";

    private JRadioButton openButton = null;
    private JRadioButton closedButton = null;

    private JTextField dateField = null;

    private JTextField durationField = null;
    private JComboBox durationUnitCombo = null;

    private JTextField reminderField = null;
    private JComboBox reminderUnitCombo = null;

    private JTextArea descriptionArea = null;

    private JCheckBox periodicCheck = null;
    private JTextField periodicField = null;
    private JComboBox periodicUnitCombo = null;
    private JTextField periodicEndField = null;

    private JButton okayButton = null;
    private JButton cancelButton = null;

    private JButton addUButton = null;
    private JButton addAllUButton = null;
    private JButton removeUButton = null;
    private JButton removeAllUButton = null;

    private boolean result = false;
    protected DateFormat dateFormat = null;

    private final UserJList leftUserList = new UserJList();
    private final UserJList rightUserList = new UserJList();

    private HashSet leftUserSet = new HashSet();
    private HashSet rightUserSet = new HashSet();
    private UserDB userDB = null;
    private User currentUser = null;

    /**
     * Construct a dialog.
     */
    public AppointmentDialog(Frame parent, UserDB userDB, User crtUser)
    {
        super(parent, "Appointment Info", true);
		this.userDB = userDB;
		currentUser = crtUser;

        // Dialog is not resizable.
        setResizable(false);

        // Set close handler for window.
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event)
            {
                doCancel();
            }
        });

        // Get instances of date formatter.
        dateFormat = DateFormat.getInstance();

        // Layout components.
        JPanel appointmentPanel = new JPanel(new BorderLayout());
        appointmentPanel.add(createEditPanel(), BorderLayout.CENTER);
        appointmentPanel.add(createButtonPanel(), BorderLayout.SOUTH);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(appointmentPanel, BorderLayout.CENTER);
        getContentPane().add(createSharedPanel(), BorderLayout.EAST);
        pack();

        // Initially select closed button.
        closedButton.setSelected(true);
    }

    /**
     * Return the result of the dialog.
     */
    public boolean isOkay()
    {
        return result;
    }

    /**
     * Add a user to the Shared Appointment User List.
     */
    public void addUserToAppointment(String email)
    {
		// add to appointment
    	rightUserSet.add(email);
		rightUserList.refreshList(rightUserSet.toArray());

		// remove from userlist (LEFT)
		leftUserSet.remove(email);
		leftUserList.refreshList(leftUserSet.toArray());
    }

    /**
     * Remove a user from the Shared Appointment User List.
     */
    public void removeUserFromAppointment(String email)
    {
		// add to userlist (LEFT)
    	leftUserSet.add(email);
		leftUserList.refreshList(leftUserSet.toArray());

		// remove from appointment
		rightUserSet.remove(email);
    	rightUserList.refreshList(rightUserSet.toArray());
	}

    /**
     * Set the unit of the input.
     */
    public String setUnit(int i)
    {
        if (i == Calendar.MINUTE)
            return MINUTES;
        if (i == Calendar.HOUR_OF_DAY)
            return HOURS;
        if (i == Calendar.DAY_OF_YEAR)
            return DAYS;
        if (i == Calendar.WEEK_OF_YEAR)
            return WEEKS;
        if (i == Calendar.MONTH)
            return MONTHS;

        return YEARS;
    }

    /**
     * Return the unit of the input.
     */
    public int getUnit(String s)
    {
        if (s.equals(MINUTES))
            return Calendar.MINUTE;
        if (s.equals(HOURS))
            return Calendar.HOUR_OF_DAY;
        if (s.equals(DAYS))
            return Calendar.DAY_OF_YEAR;
        if (s.equals(WEEKS))
            return Calendar.WEEK_OF_YEAR;
        if (s.equals(MONTHS))
            return Calendar.MONTH;

        return Calendar.YEAR;
    }

    /**
     * Return the type of the appointment.
     */
    public int getType()
    {
        if (openButton.isSelected())
            return OPEN_TYPE;
        return CLOSED_TYPE;
    }

    /**
     * Set the type of the appointment.
     */
    public void setType(int i)
    {
        if (i == OPEN_TYPE)
            openButton.setSelected(true);
        else
            closedButton.setSelected(true);
    }

    /**
     * Return the date of the appointment.
     */
    public Date getDate()
    {
        Date date = null;
        // Parse the date.
        try {
            date = dateFormat.parse(dateField.getText());
        } catch (ParseException ex) {
            date = new Date();
        }

        return date;
    }

    /**
     * Set the date of the appointment.
     */
    public void setDate(Date date)
    {
        dateField.setText(dateFormat.format(date));
    }

    /**
     * Return the duration of the appointment.
     */
    public int getDuration()
    {
        try {
            return Integer.valueOf(durationField.getText()).intValue();
        } catch (NumberFormatException ex) {
        }
        return 0;
    }

    /**
     * Set the duration of the appointment.
     */
    public void setDuration(int i)
    {
        durationField.setText(Integer.toString(i));
    }

    /**
     * Return the duration unit of the appointment.
     */
    public int getDurationUnit()
    {
        return getUnit(durationUnitCombo.getSelectedItem().toString());
    }

    /**
     * Set the duration unit of the appointment.
     */
    public void setDurationUnit(int i)
    {
		durationUnitCombo.setSelectedItem(setUnit(i));
    }

    /**
     * Return the reminder time befor the appointment.
     */
    public int getReminderTime()
    {
        try {
            return Integer.valueOf(reminderField.getText()).intValue();
        } catch (NumberFormatException ex) {
        }
        return 0;
    }

    /**
     * Set the reminder time befor the appointment.
     */
    public void setReminderTime(int i)
    {
        reminderField.setText(Integer.toString(i));
    }

    /**
     * Return the reminder time unit of the appointment.
     */
    public int getReminderTimeUnit()
    {
        return getUnit(reminderUnitCombo.getSelectedItem().toString());
    }

    /**
     * Set the reminder time unit of the appointment.
     */
    public void setReminderTimeUnit(int i)
    {
		reminderUnitCombo.setSelectedItem(setUnit(i));
    }

    /**
     * Return the description of the appointment.
     */
    public String getDescription()
    {
        return descriptionArea.getText();
    }

    /**
     * Set the description of the appointment.
     */
    public void setDescription(String s)
    {
        descriptionArea.setText(s);
    }

    /**
     * Return the period of the appointment.
     */
    public boolean getPeriodicAppointment()
    {
        return periodicCheck.isSelected();
    }

    /**
     * Set the period of the appointment.
     */
    public void setPeriodicAppointment(boolean s)
    {
        periodicCheck.setSelected(s);
        periodicCheck.setEnabled(false);
    }

    /**
     * Return the period of the appointment.
     */
    public int getPeriod()
    {
        try {
            return Integer.valueOf(periodicField.getText()).intValue();
        } catch (NumberFormatException ex) {
        }
        return 0;
    }

    /**
     * Set the period of the appointment.
     */
    public void setPeriod(int i)
    {
        periodicField.setText(Integer.toString(i));
    }

    /**
     * Return the period unit of the appointment.
     */
    public int getPeriodUnit()
    {
        return getUnit(periodicUnitCombo.getSelectedItem().toString());
    }

    /**
     * Set the period unit of the appointment.
     */
    public void setPeriodUnit(int i)
    {
		periodicUnitCombo.setSelectedItem(setUnit(i));
    }

    /**
     * Return the period of the appointment.
     */
    public Date getPeriodEnd()
    {
        Date date = null;
        // Parse the date.
        try {
            date = dateFormat.parse(periodicEndField.getText());
        } catch (ParseException ex) {
            date = new Date();
        }

        return date;
    }

    /**
     * Set the period of the appointment.
     */
    public void setPeriodEnd(Date date)
    {
        periodicEndField.setText(dateFormat.format(date));
    }


    /**
     * @Returns Participant List.
     */
    public HashSet getParticipantList(){
    	rightUserSet.add(currentUser.getEmail());
 		return rightUserSet;
    }

    /**
     * Set the Participant List.
     */
    public void setParticipantList(HashSet pl){
		// set the Appointments Participant List
		rightUserSet = pl;
		rightUserSet.remove(currentUser.getEmail());
		rightUserList.refreshList(rightUserSet.toArray());

		// remove the Appointments Participant List from Users List (LEFT)
		Object[] tmpUsr = userDB.getUserArray();
		leftUserSet = new HashSet();
		for(int i=0;i<tmpUsr.length;i++){
			leftUserSet.add(((User)tmpUsr[i]).getEmail());
//			System.out.println(tmpUsr[i]);
			}
		leftUserSet.remove(currentUser.getEmail());

		Iterator it = rightUserSet.iterator();
		while (it.hasNext())
			leftUserSet.remove(it.next());
		leftUserList.refreshList(leftUserSet.toArray());
    }


    /**
     * Closes the dialog with "true" result.
     */
    protected void doOkay()
    {
        result = true;
        setVisible(false);
        dispose();
    }

    /**
     * Closes the dialog with "false" result.
     */
    protected void doCancel()
    {
        result = false;
        setVisible(false);
        dispose();
    }

    /**
     * Creates the edit panel.
     */
    protected JPanel createEditPanel()
    {

        JPanel panel = new JPanel();

        // Set layout manager.

        GridBagLayout grid = new GridBagLayout();
        panel.setLayout(grid);

        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(2, 2, 2, 2);

        // Layout components.

        JLabel label = null;
        JScrollPane scroll = null;

        // Layout radio buttons.
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Type"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(openButton = new JRadioButton("Open"), c);
        panel.add(openButton);

        c.gridx = 2;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(closedButton = new JRadioButton("Closed"), c);
        panel.add(closedButton);

        c.gridx = 3;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(periodicCheck = new JCheckBox("Periodic"), c);
        panel.add(periodicCheck);

        ButtonGroup group = new ButtonGroup();
        group.add(openButton);
        group.add(closedButton);

        // Layout date field.
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Date/time"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(dateField = new JTextField(10), c);
        panel.add(dateField);

        // Layout duration field.
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Duration"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(durationField = new JTextField(10), c);
        panel.add(durationField);

        c.gridx = 3;
        c.gridy = 2;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(durationUnitCombo = new JComboBox(), c);
        panel.add(durationUnitCombo);

        durationUnitCombo.addItem(MINUTES);
        durationUnitCombo.addItem(HOURS);
        durationUnitCombo.addItem(DAYS);

        // Layout reminder field.
        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Reminder"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 3;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(reminderField = new JTextField(10), c);
        panel.add(reminderField);

        reminderField.setEditable(true);

        c.gridx = 3;
        c.gridy = 3;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(reminderUnitCombo = new JComboBox(), c);
        panel.add(reminderUnitCombo);

        reminderUnitCombo.addItem(MINUTES);
        reminderUnitCombo.addItem(HOURS);
        reminderUnitCombo.addItem(DAYS);

        reminderUnitCombo.setEnabled(true);

        // Description text area.
        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Description"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 4;
        c.gridwidth = 3;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(scroll = new JScrollPane(descriptionArea = new JTextArea(3, 25)), c);
        panel.add(scroll);

        descriptionArea.setWrapStyleWord(true);
        descriptionArea.setLineWrap(true);

        // Layout period field.
        c.gridx = 0;
        c.gridy = 5;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Period"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 5;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(periodicField = new JTextField(10), c);
        panel.add(periodicField);

        periodicField.setEnabled(false);

        c.gridx = 3;
        c.gridy = 5;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(periodicUnitCombo = new JComboBox(), c);
        panel.add(periodicUnitCombo);

        periodicUnitCombo.addItem(DAYS);
        periodicUnitCombo.addItem(WEEKS);
        periodicUnitCombo.addItem(MONTHS);
        periodicUnitCombo.addItem(YEARS);

        periodicUnitCombo.setEnabled(false);

        // Layout period End field.
        c.gridx = 0;
        c.gridy = 6;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.EAST;
        grid.setConstraints(label = new JLabel("Final date"), c);
        panel.add(label);

        c.gridx = 1;
        c.gridy = 6;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.WEST;
        grid.setConstraints(periodicEndField = new JTextField(10), c);
        panel.add(periodicEndField);




        periodicEndField.setEnabled(false);

        // Add some action listeners.
        openButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                durationUnitCombo.setEnabled(false);
                durationUnitCombo.setSelectedItem(DAYS);
            }
        });

        closedButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                durationUnitCombo.setEnabled(true);
                durationUnitCombo.setSelectedItem(MINUTES);
            }
        });

        periodicCheck.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                periodicField.setEnabled(periodicCheck.isSelected());
                periodicUnitCombo.setEnabled(periodicCheck.isSelected());
				periodicEndField.setEnabled(periodicCheck.isSelected());
            }
        });

        // Add some focus listeners.
        dateField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent event)
            {
            }
            public void focusLost(FocusEvent event)
            {
                if (event.isTemporary())
                    return;

                // Parse and format the date.
                try {
                    Date date = dateFormat.parse(dateField.getText());
                    dateField.setText(dateFormat.format(date));
                } catch (ParseException ex) {
                    dateField.setText(dateFormat.format(new Date()));
                }
            }
        });

        return panel;
    }

    /**
     * Creates the shared UserEdit panel.
     */
    protected JPanel createSharedPanel()
    {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());

		rightUserList.refreshList(userDB.getUserArray());
		rightUserList.setFixedCellWidth(20 * 9);
		leftUserList.setFixedCellWidth(20 * 9);

		JPanel butPanel = new JPanel(new GridLayout(4, 1));

		butPanel.add(addUButton = new JButton("Add >"));
		butPanel.add(addAllUButton = new JButton("Add All >>"));
	    butPanel.add(removeUButton = new JButton("Remove <"));
	    butPanel.add(removeAllUButton = new JButton("Remove All <<"));

		addUButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
			    Object[] users = leftUserList.getSelectedValues();

			    if(users==null)
					return;
				for (int i=0;i<users.length;i++)
				    addUserToAppointment((String)users[i]);
			}
	    });

		addAllUButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
				leftUserSet = new HashSet();
				rightUserSet = new HashSet();

			    Object[] users = userDB.getUserArray();
			    if ((users == null) || (users.length == 0))
					return;
				for (int i=0;i<users.length;i++)
				    rightUserSet.add(((User)users[i]).getEmail());
				rightUserSet.remove(currentUser.getEmail());

				rightUserList.refreshList(rightUserSet.toArray());
				leftUserList.refreshList(leftUserSet.toArray());
			}
	    });

		removeUButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
			    Object[] emails = rightUserList.getSelectedValues();
			    if ((emails == null) || (emails.length == 0))
					return;
				for (int i=0;i<emails.length;i++)
				    removeUserFromAppointment((String)emails[i]);
			}
	    });

		removeAllUButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
			    rightUserSet = new HashSet();
				leftUserSet = new HashSet();

			    Object[] users = userDB.getUserArray();
			    if ((users == null) || (users.length == 0))
					return;
				for (int i=0;i<users.length;i++)
				    leftUserSet.add(((User)users[i]).getEmail());
				leftUserSet.remove(currentUser.getEmail());

				rightUserList.refreshList(rightUserSet.toArray());
				leftUserList.refreshList(leftUserSet.toArray());
			}
	    });

		panel.add(new JScrollPane(leftUserList),BorderLayout.WEST);
		panel.add(butPanel,BorderLayout.CENTER);
		panel.add(new JScrollPane(rightUserList),BorderLayout.EAST);

		return panel;
	}

    /**
     * Creates the button panel.
     */
    protected JPanel createButtonPanel()
    {
        JPanel panel = new JPanel(new GridLayout(1, 2));
        panel.add(okayButton = new JButton("Okay"));
        panel.add(cancelButton = new JButton("Cancel"));

        // Add some action listeners.
        okayButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                doOkay();
            }
        });

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                doCancel();
            }
        });

        return panel;
    }
}
