package coolCalendar;

import java.util.*;
import java.io.*;
import javax.swing.filechooser.*;

/**
 * The database of appointments.
 */

public class CalendarDB
{
    // sorted set of open and closed events
    protected TreeSet openAppointmentsTS   = new TreeSet();
    protected TreeSet closedAppointmentsTS = new TreeSet();

    // some hashmaps to the same events
    protected HashMap openAppointmentsHM   = new HashMap();
    protected HashMap closedAppointmentsHM = new HashMap();

	/** the default file for persistence (see Definition.java) */
    private String FileName = null; 


    public CalendarDB(String fn)
    {
		FileName=fn;
    }


    /**
       creates a new open appointment and adds it to the database.
       @return the identifier of the object (a positive integer)
     */
    Integer addOpenAppointment(OpenAppointment openApp)
    {
		addOpenAppointmentDB(openApp);
		return openApp.getKey();
    }

    /**
       creates a new open appointment with a given key and adds it to 
       the database. 

       @return the identifier of the object (a positive integer)
     */
    private void addOpenAppointment(OpenAppointment openApp, Integer key)
    {
		openApp.setKey(key);
		addOpenAppointmentDB(openApp);
    }

    /** inserts the appointment into the data base.
     */
    private void addOpenAppointmentDB(Appointment nw)
    {
		if (openAppointmentsTS.add(nw))
		    { 
			openAppointmentsHM.put(nw.getKey(),nw);
		    }
		else
		    { // shouldn't hapen
			System.out.println("Could not add open event!");
			System.exit(5);
		    }
    }



    /**
       creates a new closed appointment and adds it onto the database.

       @return the positive identifier of the object or -1 (wrapped as Integer) 
               if this event would overlap with another closed event.
     */
    Integer addClosedAppointment(ClosedAppointment cevt)
    {
		if (addClosedAppointmentDB(cevt))
		    {
			return cevt.getKey();
		    }
		else
		    {
			return new Integer(-1);
		    }
    }

    /**
       creates a new closed appointment with the given key and adds it onto the 
       database if there is no overlap.

       @return false if there is an overlap
     */
    private boolean addClosedAppointment(ClosedAppointment cevt, Integer key)
    {
		cevt.setKey(key);

		if (addClosedAppointmentDB(cevt))
		    {
			return true;
		    }
		else
		    {
			return false;
		    }
    }

    /** inserts the closed appointment into the database.
	@returns false if there is an overlap, otherwise true.
    */
    private boolean addClosedAppointmentDB(ClosedAppointment cevt)
    {
	// if it is the first, there's no prob
	if (closedAppointmentsTS.size() == 0)
	    { 
		closedAppointmentsTS.add(cevt);
		closedAppointmentsHM.put(cevt.getKey(),cevt);
		return true;
	    }
	try {
	    // check for overlap with the earlier event
	    SortedSet before = closedAppointmentsTS.headSet(cevt);
	    if (before.size()>0)
		{
		    ClosedAppointment earlE = (ClosedAppointment)
			before.last();
		    if (earlE.getEndDate().compareTo(cevt.getStartDate()) > 0)
			{ // overlap
			    //System.out.println("overlap with earlier event");
			    return false;
			}
		}
	    
	    // check for overlap with the next event
	    SortedSet after = closedAppointmentsTS.tailSet(cevt);
	    if (after.size()>0)
		{
		    ClosedAppointment nextE = (ClosedAppointment)
			after.first();
		    if (cevt.getEndDate().compareTo(nextE.getStartDate()) >0 )
			{ // overlap
			    //System.out.println("overlap with later event");
			    return false;
			}
		}
	    
	    // no overlap, insert event
	    closedAppointmentsTS.add(cevt);
	    closedAppointmentsHM.put(cevt.getKey(),cevt);
	    // finished
	    return true;
	} catch (ClassCastException ex)
	    {
		ex.printStackTrace();
		System.exit(5);
	    }
	// please the compiler
	System.out.println("You should never see this...");
	return false;
    }


    /** print first the open and then the closed events */
    public String toString()
    {
		// first the open events
		StringBuffer res = new StringBuffer("Open Events:\n");
		Iterator it = openAppointmentsTS.iterator();
		while (it.hasNext())
		    {
			OpenAppointment evt = (OpenAppointment) it.next();
			res = res.append(evt).append("\n");

		    }

		// now the closed events
		res.append("\nClosed Events:\n");
		it = closedAppointmentsTS.iterator();
		while (it.hasNext())
		    {
			ClosedAppointment evt = (ClosedAppointment) it.next();
			res = res.append(evt).append("\n");
		    }

		return res.toString();
    }

    /** returns all open events in the given time intervall
	sorted by their start date. This is a view on the original
	database, so be careful with modifications.
    */
    SortedSet getOpenAppointments(Date from,
				  Date to)
    {
		// use dummy events to get a subset
		SortedSet subs = openAppointmentsTS.subSet(new OpenAppointment(from),
							   new OpenAppointment(to));

		// now create a copy
		TreeSet res = new TreeSet();
		for(Iterator it = subs.iterator();
		    it.hasNext(); )
		    {
			OpenAppointment cce = (OpenAppointment) it.next();
			res.add(cce.copy());
		    }
		return res;
    }

    /** returns a copy of all open events in the given time intervall
	sorted by their start date. 
    */
    SortedSet getClosedAppointments(Date from,
			       Date to)
    {
		// use dummy events to get a subset
		SortedSet subs = 
		    closedAppointmentsTS.subSet(new ClosedAppointment(from),
						new ClosedAppointment(to));
		// now create a copy
		TreeSet res = new TreeSet();
		for(Iterator it = subs.iterator();
		    it.hasNext(); )
		    {
			ClosedAppointment cce = (ClosedAppointment) it.next();
			res.add(cce.copy());
		    }
		return res;
    }

    /** returns an unsorted array of copied appointments in a given range.
     */
    Appointable[] getAppointments(Date from, Date to)
    {
		//System.out.println("getApps from="+from+" to="+to);
		
		// get data from DB
		SortedSet capts = getClosedAppointments(from,to);
		SortedSet oapts = getOpenAppointments(from,to);

		//System.out.println("got "+capts.size()+" closed, "+oapts.size()+" open apps.");
		

		// create array of sufficient size
		Appointable[] res = new Appointable[capts.size()+oapts.size()];

		// copy references
		int i=0;
		for(Iterator it = oapts.iterator(); it.hasNext();i++)
		{
		    res[i] = (Appointable) it.next();
		}
		for(Iterator it = capts.iterator(); it.hasNext();i++)
		{
		    res[i] = (Appointable) it.next();
		}
		// done
		return res;
    }

    /** returns an unsorted array of copied appointments
        with active reminders in a given range.
     */
    Appointable[] getReminders(Date date)
    {
       // create array of sufficient size
	Appointable[] res = new Appointable[openAppointmentsTS.size()+closedAppointmentsTS.size()];

	Calendar tmpCalendar = Calendar.getInstance();
	
	// copy references
	int i=0;
	
	OpenAppointment openApp = null;
	Date startDate = null;
	for(Iterator it = openAppointmentsTS.iterator(); it.hasNext();) {
	    openApp = (OpenAppointment) it.next();
	    startDate = openApp.getStartDate();
	    tmpCalendar.setTime(startDate);
	    tmpCalendar.add(openApp.getReminderTimeUnit(), -openApp.getReminderTime());
	    
	    //		System.out.println("getRemind from="+tmpCalendar.getTime()+
	    //			" to="+startDate+" app:"+date);
	    
	    if ((tmpCalendar.getTime().before(date)) &&
		(startDate.after(date)))
		{
		    
		    res[i] = (Appointable) openApp;
			    i++;
		}
	}
	
	ClosedAppointment closedApp = null;
	for(Iterator it = closedAppointmentsTS.iterator(); it.hasNext();) {
	    closedApp = (ClosedAppointment) it.next();
	    startDate = closedApp.getStartDate();
	    tmpCalendar.setTime(startDate);
	    tmpCalendar.add(closedApp.getReminderTimeUnit(), -closedApp.getReminderTime());
	    
	    if ((tmpCalendar.getTime().before(date)) &&
		(startDate.after(date)))
		{
		    res[i] = (Appointable) closedApp;
		    i++;
		}
	}
	
	// create new array of minimal size
	Appointable[] finalRes = new Appointable[i];
	for (int j=0;j<i;j++)
	    finalRes[j] = res[j];
	
	// done
	return finalRes;
    }
    
    /** returns true if there are open or closed appointments in the 
	given range.
    */
    boolean hasAppointments(Date from, Date to)
    {
		// get data from DB
		SortedSet capts = getClosedAppointments(from,to);
		if (capts.size()>0)
		    return true;
		    
		SortedSet oapts = getOpenAppointments(from,to);
		if (oapts.size()>0)
		    return true;	

		// found nothing
		return false;
    }


    /*  */
    boolean hasOverlappingAppointments(ClosedAppointment cevt){
	

	// check for overlap with the earlier event
	SortedSet before = closedAppointmentsTS.headSet(cevt);
	if (before.size()>0)
	    {
		ClosedAppointment earlE = (ClosedAppointment)
		    before.last();
		if (earlE.getEndDate().compareTo(cevt.getStartDate()) > 0)
		    { // overlap
			//System.out.println("overlap with earlier event");
			return true;
		    }
	    }
	
	// check for overlap with the next event
	SortedSet after = closedAppointmentsTS.tailSet(cevt);
	if (after.size()>0)
	    {
		ClosedAppointment nextE = (ClosedAppointment)
		    after.first();
		if (cevt.getEndDate().compareTo(nextE.getStartDate()) >0 )
		    { // overlap
			//System.out.println("overlap with later event");
			return true;
		    }
	    }
	return false;
	
    }

    /** returns a copy of the open appointment with the given key. 
	@return a copy or null if there is no appointment for the key.
     */
    OpenAppointment getOpenAppointment(Integer key)
    {
		OpenAppointment app =
		    (OpenAppointment) openAppointmentsHM.get(key);
		return (app==null)?app:(app.copy());
    }


    /** returns a copy of the closed event with the given key. 
	@return a copy or null if there is no appointment for the key.
    */
    ClosedAppointment getClosedAppointment(Integer key)
    {
		ClosedAppointment app =
		    (ClosedAppointment) closedAppointmentsHM.get(key);
		return (app==null)?app:(app.copy());
    }

    /** returns an open or closed appointment for a given key. 
	@return a copy or null if there is no appointment for the key.
     */
    Appointable getAppointment(Integer key)
    {
		Appointable res = getOpenAppointment(key);
		if (res == null)
		    {
			res = getClosedAppointment(key);
		    }
		return res;
    }


    /** removes an open event with the given key. 
	@returns true if an event with that key existed, otherwise
	         false
     */
    public boolean deleteOpenAppointment(Integer key)
    {
		Object o = openAppointmentsHM.remove(key);
		if (o==null)
		    { // we have no null binding => wrong key
			return false;
		    }
		else
		    {
			openAppointmentsTS.remove(o);
			return true;
		    }
    }
    
    /** removes an open event with the given key. 
	@returns true if an event with that key existed, otherwise
	         false
     */
    public boolean deleteClosedAppointment(Integer key)
    {
		Object o = closedAppointmentsHM.remove(key);
		if (o==null)
		    { // we have no null binding => wrong key
			return false;
		    }
		else
		    {
			closedAppointmentsTS.remove(o);
			return true;
		    }
    }

    /** deletes an appointment with a given key. 

	@return true if the key exists in the data base otherwise false
     */
    public boolean deleteAppointment(Integer key)
    {
		if (openAppointmentsHM.get(key)!=null)
		    {
			return deleteOpenAppointment(key);
		    }
		else if (closedAppointmentsHM.get(key)!=null)
		    {
			return deleteClosedAppointment(key);
		    }
		else
		    {
			return false;
		    }
    }


    /** updates an open appointment from another
	open appointment which must have the same key. 
	@return true if the key exists otherwise false
     */
    boolean updateOpenAppointment(OpenAppointment nw)
    {
		if (openAppointmentsHM.get(nw.getKey())==null)
		    {
			return false;
		    }
		else
		    {
			// make a copy 
			OpenAppointment nwentry = nw.copy();

			// remove old entry
			Object o = openAppointmentsHM.remove(nw.getKey());
			openAppointmentsTS.remove(o);

			// put new entry
			openAppointmentsHM.put(nwentry.getKey(),nwentry);
			openAppointmentsTS.add(nwentry);
			return true;
		    }
    }

    /** updates a closed appointment from another
	closed appointment which must have the same key. 

	@return true if the key exists and there is no overlap with other
	        appoitnments otherwise false
     */
    boolean updateClosedAppointment(ClosedAppointment nw)
    {
		if (closedAppointmentsHM.get(nw.getKey())==null)
		    {
			return false;
		    }
		else
		    {
			// make a copy 
			ClosedAppointment nwentry = nw.copy();

			// remove and store old entry
			ClosedAppointment old = (ClosedAppointment)
			    closedAppointmentsHM.remove(nw.getKey());
			closedAppointmentsTS.remove(old);

			// try to put new entry
			if (addClosedAppointmentDB(nwentry))
			    { // ok
				return true;
			    }
			else
			    { // there is an overlap, put back the old one
				addClosedAppointmentDB(old);
				return false;
			    }
		    }
    }

    /** updates an appointment from another appointment with the same
	key. This can convert an open appointment to a closed and vice
	versa.

	@return true if there is an appointment with this key, otherwise false.
    */
    boolean updateAppointment(Appointment app)
    {
		if (app instanceof OpenAppointment)
		    {
			if (updateOpenAppointment((OpenAppointment) app))
			    {
				return true;
			    }
			else
			    {
				if (closedAppointmentsHM.get(app.getKey()) != null)
				    {
					deleteClosedAppointment(app.getKey());
					addOpenAppointmentDB(app);
					return true;
				    }
				else
				    {
					return false;
				    }
			    }
		    }
		if (app instanceof ClosedAppointment)
		    {
			if (updateClosedAppointment((ClosedAppointment) app))
			    {
				return true;
			    }
			else
			    {
				if (openAppointmentsHM.get(app.getKey()) != null)
				    {
					// delete ands store the old open appointment
					OpenAppointment old = (OpenAppointment)
					    openAppointmentsHM.get(app.getKey());
					deleteOpenAppointment(app.getKey());

					// now try to add the new closed appointment
					if (addClosedAppointmentDB(((ClosedAppointment) app).copy()))
					    { // everything fine
						return true;				    
					    }
					else
					    { // restore the old entry
						addOpenAppointmentDB(old);
						return false;
					    }
					    
				    }
				else
				    {
					return false;
				    }
			    }
		    }

		// please the compiler
		System.out.println("Internal error! You never should read this!");
		System.exit(5);
		return false;
    }

	/** save the database to a default file (see Definition.java)
	 
	 */
	public void save(){
      //Write to file 
         try { 
          FileOutputStream out = new FileOutputStream(FileName); 
          ObjectOutputStream oout = new ObjectOutputStream(out); 

          oout.writeObject(openAppointmentsTS);
          oout.writeObject(closedAppointmentsTS);
          oout.writeObject(openAppointmentsHM);
          oout.writeObject(closedAppointmentsHM);

	  oout.flush();
          out.close(); 
         } catch(java.io.IOException e) { 
           System.out.println(e.toString()); 
         } 
	}

	/** restore the database from a default file (see Definition.java)
	 
	 */
	public void restore(){
      //Read from file 
          try { 
           FileInputStream in = new FileInputStream(FileName); 
           ObjectInputStream oin = new ObjectInputStream(in); 

           openAppointmentsTS = (TreeSet)oin.readObject(); 
           closedAppointmentsTS = (TreeSet)oin.readObject(); 
           openAppointmentsHM = (HashMap)oin.readObject(); 
           closedAppointmentsHM = (HashMap)oin.readObject(); 

           in.close(); 
          } catch(java.io.IOException e) { 
            System.out.println("Cannot read from "+FileName); 
          }  catch(java.lang.ClassNotFoundException e) { 
            System.out.println(e.toString()); 
          } 
	}
}
