package coolCalendar;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * This is a somewhat complex class that displays the Appointable
 * contents of a CalendarModel and notifies listeners when a specific
 * day is selected.
 */
public class CalendarPanel extends JPanel implements CalendarModelListener
{
    private String[] mondayWeek = { "Mo", "Tue", "We", "Thu", "Fr", "Sa", "Su" };
    private String[] sundayWeek = { "Su", "Mo", "Tue", "We", "Thu", "Fr", "Sa" };
    private String[] week = null;
    private EventListenerList listenerList = new EventListenerList();

    // Current calendar (date) to display.
    private Calendar calendar = null;

    // The calendar model to display.
    private CalendarModel model = null;

    // Some borders.
    private Border normalBorder = null;
    private Border busyBorder = null;

    // Pool of buttons to re-use.
    private JButton[] buttonPool = null;

    /**
     * Constructs a CalendarPanel with a specific calendar and
     * calendar model.
     */
    public CalendarPanel(Calendar c, CalendarModel model)
    {
        super();

        // Save the model.
        this.model = model;

        // Subscribe for calendar model events.
        model.addCalendarModelListener(this);

        // Create border for busy days.
        busyBorder = BorderFactory.createLineBorder(Color.blue, 2);
        normalBorder = new JButton().getBorder();

        // Create button pool for gui.
        buttonPool = new JButton[31];
        for (int i = 0; i < buttonPool.length; i++)
            buttonPool[i] = new JButton();

        // Create and set action listener for buttons.
        ActionListener al = new ActionListener() {
            public void actionPerformed(ActionEvent event)
            {
                int day = Integer.valueOf(((JButton) event.getSource()).getText()).intValue();
                calendar.set(Calendar.DAY_OF_MONTH, day);
                fireChangeEvent();
            }
        };

        for (int i = 0; i < buttonPool.length; i++)
            buttonPool[i].addActionListener(al);

        // Set layout manager for panel.
        setLayout(new GridLayout(7, 7));

        // Finally set the calendar.
        setCalendar(c);
    }

    /**
     * Returns the calendar model.
     */
    public CalendarModel getModel()
    {
        return model;
    }

    /**
     * Returns the current calendar.
     */
    public Calendar getCalendar()
    {
        return (Calendar) calendar.clone();
    }

    /**
     * Sets the current calendar and will change the
     * display accordingly.
     */
    public void setCalendar(Calendar newCalendar)
    {
        // Reset the current day to black if it
        // was previously red.
        Calendar today = Calendar.getInstance();
        if ((calendar != null) &&
            ((today.get(Calendar.MONTH) == calendar.get(Calendar.MONTH)) &&
            (today.get(Calendar.YEAR) == calendar.get(Calendar.YEAR))))
        {
            int day = today.get(Calendar.DAY_OF_MONTH);
            buttonPool[day - 1].setForeground(Color.black);
        }

        calendar = newCalendar;

        // Remove all child components.
        removeAll();

        // Choose correct heading based on first day of week.
        if (calendar.getFirstDayOfWeek() == Calendar.SUNDAY)
            week = sundayWeek;
        else
            week = mondayWeek;

        // Display headings.
        for (int i = 0; i < week.length; i++)
            add(new JLabel(week[i], JLabel.CENTER));

        // Get some values for building calendar display.
        int leading = calculateLeadingDays(calendar);
        int trailing = calculateTrailingDays(calendar);
        int daysInMonth = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);

        // Display leading blanks.
        for (int i = 0; i < leading; i++)
            add(new JLabel());

        // Display day buttons.
        for (int i = 0; i < daysInMonth; i++)
        {
            buttonPool[i].setText(Integer.toString(i + 1));
            add(buttonPool[i]);
        }

        // Display trailing blanks.
        for (int i = 0; i < trailing; i++)
            add(new JLabel());

        // Set the current day to red.
        if ((today.get(Calendar.MONTH) == calendar.get(Calendar.MONTH))
        	&& (today.get(Calendar.YEAR) == calendar.get(Calendar.YEAR)))
        {
            int day = today.get(Calendar.DAY_OF_MONTH);
            buttonPool[day - 1].setForeground(Color.red);
        }

        // Update busy days.
        Calendar c = getCalendarForFirstDayOfMonth(calendar);
        for ( ; c.get(Calendar.MONTH) == calendar.get(Calendar.MONTH); c.add(Calendar.DAY_OF_MONTH, 1))
        {
            int day = c.get(Calendar.DAY_OF_MONTH);

            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            Date from = c.getTime();
            c.set(Calendar.HOUR_OF_DAY, 23);
            c.set(Calendar.MINUTE, 59);
            c.set(Calendar.SECOND, 59);
            Date to = c.getTime();

            if (model.hasAppointments(from, to))
                setBusyDay(day, true);
            else
                setBusyDay(day, false);
        }

        revalidate();

        fireChangeEvent();
    }

    /**
     * Returns the currently selected day as a number.
     */
    public int getSelectedDay()
    {
        return calendar.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * Changes the current day, no attempt is made to validate.
     */
    public void setSelectedDay(int i)
    {
        calendar.set(Calendar.DAY_OF_MONTH, i);
        fireChangeEvent();
    }

    /**
     * Checks to see if a day is busy.
     */
    public boolean isBusyDay(int day)
    {
        if (buttonPool[day - 1].getBorder() == busyBorder)
            return true;
        return false;
    }

    /**
     * Sets a day as busy or not by changing its border.
     */
    public void setBusyDay(int day, boolean b)
    {
        buttonPool[day - 1].setBorder(b ? busyBorder : normalBorder);
    }

    /**
     * Simple utility function to get first day of month
     * for a given calendar.  For example, if it is given
     * a calendar for "April 15, 2000" it will return a
     * calendar for "April 1, 2000".
     */
    protected Calendar getCalendarForFirstDayOfMonth(Calendar calendar)
    {
        Calendar c = (Calendar) calendar.clone();
        int day = c.get(Calendar.DAY_OF_MONTH);
        c.add(Calendar.DAY_OF_MONTH, -(day - 1));
        return c;
    }

    /**
     * Simple utility function to calculate the number of
     * leading blank days to display on the calendar.
     */
    protected int calculateLeadingDays(Calendar calendar)
    {
        Calendar c = getCalendarForFirstDayOfMonth(calendar);
        int dayOfWeek = c.get(Calendar.DAY_OF_WEEK);

        if (dayOfWeek == c.getFirstDayOfWeek())
            return 0;

        int count = 0;
        for ( ; c.get(Calendar.DAY_OF_WEEK) != c.getFirstDayOfWeek(); c.add(Calendar.DAY_OF_YEAR, -1))
            count++;

        return count;
    }

    /**
     * Simple utility function to calculate the number of
     * trailing blank days to display on the calendar.
     */
    protected int calculateTrailingDays(Calendar calendar)
    {
        int leading = calculateLeadingDays(calendar);
        int daysInMonth = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
        return 42 - (leading + daysInMonth);
    }

    /**
     * CalendarModelListener interfaces; these following interfaces
     * listen for changes in the CalendarModel and update the display
     * accordingly.
     */
    public void appointmentAdded(CalendarModelEvent event)
    {
        Appointable appointable = event.getAppointment();
        Calendar c = Calendar.getInstance();
        c.setTime(appointable.getStartDate());
        if (c.get(Calendar.MONTH) == getCalendar().get(Calendar.MONTH))
        {
            int day = c.get(Calendar.DAY_OF_MONTH);
            setBusyDay(day, true);
        }
    }

    public void appointmentRemoved(CalendarModelEvent event)
    {
        Appointable appointable = event.getAppointment();
        Calendar c = Calendar.getInstance();
        c.setTime(appointable.getStartDate());
        if (c.get(Calendar.MONTH) == getCalendar().get(Calendar.MONTH))
        {
            int day = c.get(Calendar.DAY_OF_MONTH);

            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            Date from = c.getTime();
            c.set(Calendar.HOUR_OF_DAY, 23);
            c.set(Calendar.MINUTE, 59);
            c.set(Calendar.SECOND, 59);
            Date to = c.getTime();

            if (model.hasAppointments(from, to))
                setBusyDay(day, true);
            else
                setBusyDay(day, false);
        }
    }

    public void appointmentUpdated(CalendarModelEvent event)
    {
        // Since we don't know the previous date of the
        // last appointment we need to update the entire
        // month's busy days just in case.
        Calendar c = getCalendarForFirstDayOfMonth(getCalendar());
        for ( ; c.get(Calendar.MONTH) == getCalendar().get(Calendar.MONTH); c.add(Calendar.DAY_OF_MONTH, 1))
        {
            int day = c.get(Calendar.DAY_OF_MONTH);

            c.set(Calendar.HOUR_OF_DAY, 0);
            c.set(Calendar.MINUTE, 0);
            c.set(Calendar.SECOND, 0);
            Date from = c.getTime();
            c.set(Calendar.HOUR_OF_DAY, 23);
            c.set(Calendar.MINUTE, 59);
            c.set(Calendar.SECOND, 59);
            Date to = c.getTime();

            if (model.hasAppointments(from, to))
                setBusyDay(day, true);
            else
                setBusyDay(day, false);
        }
    }

    /**
     * ChangeListener event handlers; these interfaces
     * keep track of other objects listening to us.
     */
    public void addChangeListener(ChangeListener l)
    {
        listenerList.add(ChangeListener.class, l);
    }

    public void removeChangeListener(ChangeListener l)
    {
        listenerList.remove(ChangeListener.class, l);
    }

    protected void fireChangeEvent()
    {
        ChangeEvent changeEvent = null;

        // Guaranteed to return a non-null array

        Object[] listeners = listenerList.getListenerList();

        // Process the listeners last to first, notifying
        // those that are interested in this event

        for (int i = listeners.length - 2; i >= 0; i -= 2)
        {
            if (listeners[i] == ChangeListener.class)
            {
                // Lazily create the event:

                if (changeEvent == null)
                    changeEvent = new ChangeEvent(this);

                ((ChangeListener) listeners[i+1]).stateChanged(changeEvent);
            }
        }
    }
}
