package coolCalendar;

import java.util.*;

public class MultiCalendarModel 
    extends MutableCalendarModel
    implements CalendarModel
{


    private User myUser=null;
    private SharedCalendarDB sharedDB=null;
    private UserDB userDB;

    /** creates a multi calendar model for a given calendar data base.
     */
    public MultiCalendarModel(UserDB userDB,User user)
    {
	super(user.getCalendar().getCalendarDB());
	myUser = user;
	sharedDB = userDB.getSharedCalendar();
	this.userDB = userDB;
    }

    public User getUser(){
	return myUser;
    }

    /** returns true if the user has appointments
     */
    public boolean hasAppointments(Date from, Date to)
    {
        return myDB.hasAppointments(from,to) || sharedDB.hasAppointments(from,to,myUser) ;
    }

    /** returns true if the user has appointments - used when trying to add closed app
     */
    public boolean hasOverlappingAppointments(ClosedAppointment app,User user){

	CalendarDB cdb = user.getCalendar().getCalendarDB();

	//SortedSet res1=cdb.getClosedAppointments(from,to);
	if(cdb.hasOverlappingAppointments(app))
	    return true;
	//res1=sharedDB.getClosedAppointments(from,to,user);
	if(sharedDB.hasOverlappingAppointments(app,user))
	    return true;
	return false;
    }

    /** returns true if all participants in a closed appointment are free
     */

    public boolean allParticipantsFree(ClosedAppointment cevt){
	HashSet part=cevt.getParticipantList();
	

	for(Iterator it=part.iterator();it.hasNext();){
	  
	    String email=(String)it.next();
	    User user=userDB.getUser(email);
	    if(user==null)
		return false;
	    if(hasOverlappingAppointments(cevt,user))
		return false;

	}
	return true;
    }


    public Appointable[] getAppointments(Date from, Date to)
    {
        Appointable[] res1=myDB.getAppointments(from,to);
        Appointable[] res2=sharedDB.getAppointments(from,to,myUser);

        Appointable[] res= new Appointable[res1.length+res2.length];
        int i;
	for(i=0;i<res1.length;i++)
	    res[i]=res1[i];
	for(i=0;i<res2.length;i++)
	    res[i+res1.length]=res2[i];
	return res;
    }

    public Appointable[] getReminders(Date date)
    {
        Appointable[] res1=myDB.getReminders(date);
        Appointable[] res2=sharedDB.getReminders(date);

        Appointable[] res= new Appointable[res1.length+res2.length];
        int i;
	for(i=0;i<res1.length;i++)
	    res[i]=res1[i];
	for(i=0;i<res2.length;i++)
	    res[i+res1.length]=res2[i];
	return res;
    }

    /**
       creates a new open appointment and adds it to the calendar.
       @return the identifier of the object (a positive integer)
    */
    Integer addOpenAppointment(OpenAppointment oapp)
    {
	Integer key;
	if(oapp.isShared()){
	    key = sharedDB.addOpenAppointment(oapp);  
	    if (key.intValue() != -1)
		{ // the appointment is actually added
		    fireCalendarModelAdded(sharedDB.getAppointment(key));
		}
	}
	else{
	    key = myDB.addOpenAppointment(oapp);
	    if (key.intValue() != -1)
		{ // the appointment is actually added
		    fireCalendarModelAdded(myDB.getAppointment(key));
		}
	}
	return key;
    }

    /**
       creates a new closed appointment and adds it to the calendar.

       @return the positive identifier of the object or -1 (wrapped as Integer) 
       if this event would overlap with another closed event.
    */
    Integer addClosedAppointment(ClosedAppointment cevt)
    {
	
	Integer key;
	if(cevt.isShared()){
	    //		System.out.println("shared ");
	    if(!allParticipantsFree(cevt))
		return new Integer(-1);
		


	    key = sharedDB.addClosedAppointment(cevt);  
	    if (key.intValue() != -1)
		{ // the appointment is actually added
		    fireCalendarModelAdded(sharedDB.getAppointment(key));
		}
	} 
	else {
	    // only shared check !
	    /*
	      if(hasClosedAppointments(cevt.getStartDate(),cevt.getEndDate(),myUser))
	      return new Integer(-1);
	    */

	    if(sharedDB.hasOverlappingAppointments(cevt,myUser))
		return new Integer(-1);

	    key = myDB.addClosedAppointment(cevt);
	    if (key.intValue() != -1)
		{ // the appointment is actually added
		    fireCalendarModelAdded(myDB.getAppointment(key));
		}
	}
	    
	return key;
    }

    /** returns an open or closed appointment for a given key. 
	@return a copy or null if there is no appointment for the key.
    */
    /* not used*/
    Appointable getAppointment(Integer key)
    {
	return myDB.getAppointment(key);
    }
    

    
    /** deletes an appointment with a given key. 

	@return true if the key exists in the data base otherwise false
    */
    public boolean deleteAppointment(Appointment app)
    {
	boolean res;
	 
	if(app.isShared()){
	    res = sharedDB.deleteAppointment(app.getKey());
	}
	else{
	    res = myDB.deleteAppointment(app.getKey());
	}
		
	if (res)
	    {
		fireCalendarModelRemoved(app);
	    }
	return res;
    }

    /** updates an appointment from another appointment with the same
	key. This can convert an open appointment to a closed and vice
	versa.

	@return true if there is an appointment with this key, otherwise false.
    */
    boolean updateAppointment(Appointment app)
    {
	boolean res;

	if(app.isShared()){

	    if(app instanceof ClosedAppointment){

		ClosedAppointment old = sharedDB.getClosedAppointment(app.getKey());
		if (old != null) {
		    res=sharedDB.deleteAppointment(old.getKey());

		    if(sharedDB.hasOverlappingAppointments((ClosedAppointment)app,myUser)
		       || myDB.hasOverlappingAppointments((ClosedAppointment)app))
			{
			    sharedDB.addClosedAppointment(old);
			    return false;
			}
		} else {
		    old = myDB.getClosedAppointment(app.getKey());
		    res = myDB.deleteAppointment(old.getKey());

		    if(sharedDB.hasOverlappingAppointments((ClosedAppointment)app,myUser)
		       || myDB.hasOverlappingAppointments((ClosedAppointment)app))
			{
			    myDB.addClosedAppointment(old);
			    return false;
			}
		}
		sharedDB.addClosedAppointment((ClosedAppointment)app);
	    } else {
		OpenAppointment old = sharedDB.getOpenAppointment(app.getKey());
		if (old != null)
		    res=sharedDB.deleteAppointment(old.getKey());
		else
		    res=myDB.deleteAppointment(old.getKey());

		sharedDB.addOpenAppointment((OpenAppointment)app);

		if (res)
		    {
			fireCalendarModelUpdated(sharedDB.getAppointment(app.getKey()));
		    }
	    }
	} else {
	    ClosedAppointment old = myDB.getClosedAppointment(app.getKey());
	    if (old != null) {
		res=myDB.deleteAppointment(old.getKey());
		    
		if(sharedDB.hasOverlappingAppointments((ClosedAppointment)app,myUser)
		   || myDB.hasOverlappingAppointments((ClosedAppointment)app))
		    {
			myDB.addClosedAppointment(old);
			return false;
		    }
	    } else {
		old = sharedDB.getClosedAppointment(app.getKey());
		res = sharedDB.deleteAppointment(old.getKey());

		if(sharedDB.hasOverlappingAppointments((ClosedAppointment)app,myUser)
		   || myDB.hasOverlappingAppointments((ClosedAppointment)app))
		    {
			sharedDB.addClosedAppointment(old);
			return false;
		    }
	    }
	    myDB.addClosedAppointment((ClosedAppointment)app);

	    if (res)
		{
		    fireCalendarModelUpdated(myDB.getAppointment(app.getKey()));
		}
	}
	return res;
    }
    
}
