package coolCalendar;

import java.util.*;


public class SharedCalendarDB extends CalendarDB
{

    private UserDB userDB;

    public SharedCalendarDB(String fn,UserDB userDB){
	super(fn);
	this.userDB=userDB;
    }

    /** returns all open events in the given time intervall
	sorted by their start date. This is a view on the original
	database, so be careful with modifications.
    */
    SortedSet getOpenAppointments(Date from,
				  Date to, User user)
    {
		// use dummy events to get a subset
		SortedSet subs = openAppointmentsTS.subSet(new OpenAppointment(from),
							   new OpenAppointment(to));

		// now create a copy
		TreeSet res = new TreeSet();
		for(Iterator it = subs.iterator();
		    it.hasNext(); )
		    {
			OpenAppointment cce = (OpenAppointment) it.next();
			if(cce.hasUser(user))
			  res.add(cce.copy());
		    }
		return res;
    }


    /** returns a copy of all closed events in the given time intervall
	sorted by their start date. 
    */
    
    SortedSet getClosedAppointments(Date from,
			       Date to, User user)
    {
		// use dummy events to get a subset
		SortedSet subs = 
		    closedAppointmentsTS.subSet(new ClosedAppointment(from),
						new ClosedAppointment(to));
		// now create a copy
		TreeSet res = new TreeSet();
		for(Iterator it = subs.iterator();
		    it.hasNext(); )
		    {
			ClosedAppointment cce = (ClosedAppointment) it.next();
			if(cce.hasUser(user))
			   res.add(cce.copy());
		    }
		return res;
    }



 /** returns an unsorted array of copied appointments in a given range.
     */

    Appointable[] getAppointments(Date from, Date to, User user)
    {
		//System.out.println("getApps from="+from+" to="+to);

		// get data from DB
		SortedSet capts = getClosedAppointments(from,to,user);
		SortedSet oapts = getOpenAppointments(from,to,user);

		//System.out.println("got "+capts.size()+" closed, "+oapts.size()+" open apps.");

		// create array of sufficient size
		Appointable[] res = new Appointable[capts.size()+oapts.size()];

		// copy references
		int i=0;
		for(Iterator it = oapts.iterator(); it.hasNext();i++)
		{
		    res[i] = (Appointable) it.next();
		}
		for(Iterator it = capts.iterator(); it.hasNext();i++)
		{
		    res[i] = (Appointable) it.next();
		}
		// done
		return res;
    }


    public boolean hasAppointments(Date from, Date to,User user){
	// get data from DB
	SortedSet capts = getClosedAppointments(from,to,user);
	if(capts.size()>0)
	    return true;
	SortedSet oapts = getOpenAppointments(from,to,user);
	if(oapts.size()>0)
	    return true;
	return false;
    }

    public boolean hasOverlappingAppointments(ClosedAppointment cevt,
					      User user){

	// check for overlap with the earlier event
	SortedSet before = closedAppointmentsTS.headSet(cevt);

	if(before.size()>0)
	    while( !((ClosedAppointment)before.last()).getParticipantList().contains(user.getEmail()) ){
		before.remove(before.last());
		if(before.size()==0)
		    break;
	    }

	if (before.size()>0)
	    {
		ClosedAppointment earlE = (ClosedAppointment)
		    before.last();
		if (earlE.getEndDate().compareTo(cevt.getStartDate()) > 0)
		    { // overlap
			//System.out.println("overlap with earlier event");
			return true;
		    }
	    }
	
	// check for overlap with the next event
	SortedSet after = closedAppointmentsTS.tailSet(cevt);

	if (after.size()>0)
	    while(!  ((ClosedAppointment)after.first()).getParticipantList().contains(user.getEmail()) ){
		after.remove(after.first());
		if(after.size()==0)
		    break;
	    }

	if (after.size()>0)
	    {
		ClosedAppointment nextE = (ClosedAppointment)
		    after.first();
		if (cevt.getEndDate().compareTo(nextE.getStartDate()) >0 )
		    { // overlap
			//System.out.println("overlap with later event");
			return true;
		    }
	    }
	return false;



    }

    
    /**
       creates a new closed appointment and adds it onto the database.

       @return the positive identifier of the object or -1 (wrapped as Integer) 
               if this event would overlap with another closed event.
     */
    Integer addClosedAppointment(ClosedAppointment cevt)
    {
		if (addClosedAppointmentDB(cevt))
		    {
			return cevt.getKey();
		    }
		else
		    {
			return new Integer(-1);
		    }
    }


    /**
       creates a new closed appointment with the given key and adds it onto the 
       database if there is no overlap.

       @return false if there is an overlap
     */
    private boolean addClosedAppointment(ClosedAppointment cevt, Integer key)
    {
		cevt.setKey(key);

		if (addClosedAppointmentDB(cevt))
		    {
			return true;
		    }
		else
		    {
			return false;
		    }
    }



    /** inserts the closed appointment into the database.
	@returns false if there is an overlap, otherwise true.
    */
    private boolean addClosedAppointmentDB(ClosedAppointment cevt)
    {	
	closedAppointmentsTS.add(cevt);
	closedAppointmentsHM.put(cevt.getKey(),cevt);
	return true;
    }

}
